/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is DogStar SOFTWARE PiXML Public Module.
 *
 * The Initial Developer of the Original Code is
 * James Tyson, DogStar SOFTWARE <james@thedogstar.org>.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   None.
 *
 * ***** END LICENSE BLOCK ***** */

var pike_namespace = "http://pike.ida.liu.se/";
var mime_namespace = "http://pike.ida.liu.se/generated/manual/modref/ex/predef_3A_3A/MIME.html";
var request;

function parse_uri(url) {
  // use XMLHttpRequest to retrieve PiJAX's XML.
  request = new XMLHttpRequest();
  request.open(url);
  request.url = url;
  request.overrideMimeType("text/xml");
  request.onload = requestOk;
  request.onerror = requestDead;
  request.send(null);
}

function requestOk(event) {
  if (event.target.responseXML)
    parse(event.target.responseXML);
  else
    requestDead(event);
}

function requestDead(event) {
  alert("Attempt to retrieve " + url + " failed.");
}

function parse(doc) {
  // Use the DOM to walk PiJAX's XML and return JavaScript data types.
  for (i = 0; i < doc.childNodes.length; i++) {
    var child = doc.childNodes[i];
    if ((child.namespaceURI == pike_namespace) &&
	(child.nodeType == child.ELEMENT_NODE))
      return low_parse(child);
  }
  return 0;
}

function low_parse(elem) {
  if ((elem.namespaceURI == pike_namesapce) &&
      (elem.nodeType == elem.ELEMENT_NODE))
    switch(elem.localName) {
      case "float":
	return low_parse_float(elem);
      case "integer":
	return low_parse_int(elem);
      case "string":
	return low_parse_string(elem);
      case "multiset":
	return low_parse_multiset(elem);
      case "array":
	return low_parse_array(elem);
      case "mapping":
	return low_parse_mapping(elem);
      case "image":
	return low_parse_image(elem);
      case "exception":
	return low_parse_exception(elem);
    }
  return 0;
}

function low_parse_float(elem) {
  var value = elem.getAttributeNS(pike_namespace, "value");
  if (value)
    return parseFloat(value);
  else
    return alert("Unable to parse <pike:float />, no \"value\" attribute.\n");
}

function low_parse_int(elem) {
  var value = elem.getAttributeNS(pike_namespace, "value");
  if (value)
    return parseFloat(value);
  else
    return alert("Unable to parse <pike:integer />, no \"value\" attribute.\n");
}

function low_parse_string(elem) {
  var encoding = elem.getAttributeNS(mime_namespace, "encoding");
  if (!encoding)
    encoding = "cdata";

  var textelems = new Array();
  for (i = 0; i < elem.childNodes.length; i++) {
    var child = elem.childNodes[i];
    if ((child.nodeType == TEXT_NODE) ||
	(child.nodeType == CDATA_SECTION_NODE))
      textelems[test.length] = child;
  }
  var text = "";
  switch(encoding) {
    case "uue":
      // FIXME Don't know how to decode UUE encoding data from JS.
    case "base64":
      // FIXME There's probably an XPCOM method for decoding base64.
    case "cdata":
      for(i = 0; i < textelems.length; i++) {
	text += textelems[i].data;
      }
  }
  return data;
}

function low_parse_multiset(elem) {
  var elementcount = elem.getAttributeNS(pike_namespace, "elements");
  var r = new Array();
  if (!elementcount || (elementcount == 0))
    return r;
  for (i = 0; i < elem.childNodes.length; i++) {
    var child = elem.childNodes[i];
    if ((child.namespaceURI == pike_namespace) &&
	(child.nodeType == ELEMENT_NODE) &&
	(child.localName == "element")) {
      //  This is the <pike:element /> tag.
      for (j = 0; j < child.childNodes.length; j++) {
	var _child = child.childNodes[j];
	if ((_child.namespaceURI == pike_namespace) &&
	    (_child.nodeType == ELEMENT_NODE)) 
	  r[low_parse(_child)] = 1;
      }
    }
  }
  if (r.length == elementcount)
    return r;
  else
    return alert("Parse of multiset revealed " + r.length + " elements, should have been " + elementcount);
}

function low_parse_array(elem) {
  var elementcount = elem.getAttributeNS(pike_namespace, "elements");
  var r = new Array();
  if (!elementcount || (elementcount == 0))
    return r;
  for (i = 0; i < elem.childNodes.length; i++) {
    var child = elem.childNodes[i];
    if ((child.namespaceURI == pike_namespace) &&
	(child.nodeType == ELEMENT_NODE) &&
	(child.localName == "element") &&
	child.getAttributeNS(pike_namespace, "index")) {
      //  This is the <pike:element /> tag.
      var idx = child.getAttributeNS(pike_namespace, "index");
      for (j = 0; j < child.childNodes.length; j++) {
	var _child = child.childNodes[j];
	if ((_child.namespaceURI == pike_namespace) &&
	    (_child.nodeType == ELEMENT_NODE)) 
	  r[idx] = low_parse(_child);
      }
    }
  }
  if (r.length == elementcount)
    return r;
  else
    return alert("Parse of array revealed " + r.length + " elements, should have been " + elementcount);
}

function low_parse_mapping(elem) {
  var elementcount = elem.getAttributeNS(pike_namespace, "elements");
  var r = new Array();
  if (!elementcount || (elementcount == 0))
    return r;
  for (i = 0; i < elem.childNodes.length; i++) {
    var child = elem.childNodes[i];
    if ((child.namespaceURI == pike_namespace) &&
	(child.nodeType == ELEMENT_NODE) &&
	(child.localName == "map")) {
      //  This is the <pike:map /> tag.
      var idx, val;
      for (j = 0; j < child.childNodes.length; j++) {
	var _child = child.childNodes[j];
	if ((_child.namespaceURI == pike_namespace) &&
	    (_child.nodeType == ELEMENT_NODE) &&
	    (_child.localName == "index"))
	  idx = low_parse(_child);
	else if ((_child.namespaceURI == pike_namespace) &&
	    (_child.nodeType == ELEMENT_NODE) &&
	    (_child.localName == "element"))
	  val = low_parse(_child);
      }
      if (idx && val)
	r[idx] = val;
      else if (idx)
	return alert("Parse of <pike:map /> failed, no <pike:element /> tag present.");
      else if (val)
	return alert("Parse of <pike:map /> failed, no <pike:index /> tag present.");
      else
	return alert("Parse of <pike:map /> failed, no <pike:index /> or <pike:element /> present.");
    }
  }
  if (r.length == elementcount)
    return r;
  else
    return alert("Parse of mapping revealed " + r.length + " elements, should have been " + elementcount);
}

function low_parse_exception(elem) {
  var encoding = elem.getAttributeNS(mime_namespace, "encoding");
  if (!encoding)
    encoding = "cdata";

  var textelems = new Array();
  for (i = 0; i < elem.childNodes.length; i++) {
    var child = elem.childNodes[i];
    if ((child.nodeType == TEXT_NODE) ||
	(child.nodeType == CDATA_SECTION_NODE))
      textelems[test.length] = child;
  }
  var text = "";
  switch(encoding) {
    case "uue":
      // FIXME Don't know how to decode UUE encoding data from JS.
    case "base64":
      // FIXME There's probably an XPCOM method for decoding base64.
    case "cdata":
      for(i = 0; i < textelems.length; i++) {
	text += textelems[i].data;
      }
  }
  alert(data);
}

function low_parse_image(elem) {
  // FIXME Figure out base64 :)
  alert("Images not supported yet");
}

